@@ -1,10 +1,12 @@
 #![feature(iter_intersperse)]
+#![feature(adt_const_params)]
 
 //! **I do not recommend using this directly, please sea [rust-ad](https://crates.io/crates/rust-ad).**
 
 use std::collections::HashMap;
 
 mod derivatives;
+pub use derivatives::Type;
 
 mod dict;
 pub use dict::*;
@@ -25,6 +27,8 @@ pub const REVERSE_MODE_PREFIX: &'static str = "__rev_";
 pub const FUNCTION_PREFFIX: &'static str = "f";
 /// Prefix used for flattening binary expressions as a reciever for a method.
 pub const RECEIVER_PREFIX: &'static str = "r";
+/// Prefix used for flattening retrn statements;
+pub const RETURN_SUFFIX: &'static str = "rtn";
 
 /// Gets type of given expression (only supports literals and paths)
 pub fn expr_type(expr: &syn::Expr, type_map: &HashMap<String, String>) -> String {
